# -*- coding: utf-8 -*-
import datetime
import calendar

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.conf import settings
from django.db.models import Q

from jumbo.admin.options import ExtendibleModelAdmin, OrderedModelAdmin, SettingsAdmin
from jumbo.forms.models import AdminModelForm
from jumbo.middleware.thread_local import get_current_user

from press.models import Press, PressReview, Theme, Source, DELIVERY, Sector, Format

DELIVERY.insert(0,('', '---'))

# SEARCH
class PressSearchForm(forms.ModelForm):
    class Meta:
        model = Press
        display_fields = (
            ('date_p__gte', 'date_p__lte'),
            ('delivery_date__gte', 'delivery_date__lte'),
            'delivery',
            'source__name__icontains',
            ('press_reviews__name__icontains', 'themes__name__icontains'),
            'abstract__icontains',
            'keywords__icontains',
            'press_reviews__slug',
            ('title_abstract', 'only_title')
        )
        
    def __init__(self, *args, **kws):
        super(PressSearchForm, self).__init__(*args, **kws)
        self.fields['date_p__gte'] = forms.DateField(label=_("Press Date from"), required=False)
        self.fields['date_p__lte'] = forms.DateField(label=_("Press Date to"), required=False)
        self.fields['delivery_date__gte'] = forms.DateField(label=_("Press Delivery Date from"), required=False)
        self.fields['delivery_date__lte'] = forms.DateField(label=_("Press Delivery Date to"), required=False)
        self.fields['source__name__icontains'] = forms.CharField(label=_("Source"), required=False)
        self.fields['press_reviews__name__icontains'] = forms.CharField(label=_("Press Review"), required=False)
        self.fields['themes__name__icontains'] = forms.CharField(label=_("Theme"), required=False)
        self.fields['abstract__icontains'] = forms.CharField(label=_("Abstract"), required=False)
        self.fields['keywords__icontains'] = forms.CharField(label=_("Keywords"), required=False)
        self.fields['delivery'] = forms.ChoiceField(label=_("Delivery"), choices=DELIVERY, required=False)
        self.fields['press_reviews__slug'] = forms.CharField(label=(" "), required=False, widget=forms.HiddenInput())
        self.fields['only_title'] = forms.BooleanField(label=_("Only Title"), required=False)
        self.fields['title_abstract'] = forms.CharField(label=_("Text"), required=False)

    def change_search_filter(self, lookup_params, qs):
        if lookup_params.has_key('abstract__icontains'):
            del lookup_params['abstract__icontains']
        
        only_title = None
        title_abstract = None
        if lookup_params.has_key('only_title'):
            only_title = lookup_params['only_title']
            del lookup_params['only_title']

        if lookup_params.has_key('title_abstract'):
            title_abstract = lookup_params['title_abstract']
            del lookup_params['title_abstract']
            
        if only_title and title_abstract:
            qs = qs.filter(title__icontains=title_abstract)
        if not only_title and title_abstract:
            qs = qs.filter(
               Q(title__icontains=title_abstract) |
               Q(abstract__icontains=title_abstract)
            )
        if lookup_params.has_key('delivery'):
            delivery = lookup_params['delivery']
            if delivery == 'd':
                qs = qs.filter(delivery_date=datetime.date.today())
            elif delivery == 'w':
                # settimanale
                today = datetime.date.today()
                first_date = today - datetime.timedelta(calendar.weekday(today.year, today.month, today.day))
                last_date = first_date + datetime.timedelta(6)
                qs = qs.filter(
                    delivery_date__gte=first_date,
                    delivery_date__lte=last_date
                )               
            elif delivery == 'f':
                # quindicinale
                today = datetime.date.today()
                first_day = (today.day - 1) / 15 * 15 + 1
                first_date = datetime.date(today.year, today.month, first_day)
                qs = qs.filter(
                    delivery_date__gte=first_date,
                    delivery_date__lte=today
                )
            elif delivery == 'm':
                qs = qs.filter(
                    delivery_date__month=datetime.date.today().month,
                    delivery_date__year=datetime.date.today().year
                )
            elif delivery == 't':
                # bimestrale: 1-2, 3-4, 5-6, 7-8, 9-10, 11-12
                today = datetime.date.today()
                lday_month = calendar.monthrange(today.year, today.month)[1]
                if today.month % 2 == 0:
                    # mese pari
                    last_date = datetime.date(today.year, today.month, lday_month)
                    last_date_pm = last_date - datetime.timedelta(days=lday_month)
                    first_date = datetime.date(last_date_pm.year, last_date_pm.month, 1)
                else:
                    # mese dispari
                    first_date = datetime.date(today.year, today.month, 1)
                    first_date_pm = first_date + datetime.timedelta(days=lday_month)
                    lday_p_month = calendar.monthrange(first_date_pm.year, first_date_pm.month)[1]
                    last_date = datetime.date(first_date_pm.year, first_date_pm.month, lday_p_month)
                qs = qs.filter(
                    delivery_date__gte=first_date,
                    delivery_date__lte=last_date
                )                    
            elif delivery == 'q':
                # trimestrale: 1-2-3, 4-5-6, 7-8-9, 10-11-12
                today = datetime.date.today()
                first_month = (today.month - 1) / 3 * 3 + 1
                last_month = first_month + 2
                last_day = calendar.monthrange(today.year, last_month)[1]
                first_date = datetime.date(today.year, first_month, 1)
                last_date = datetime.date(today.year, last_month, last_day)
                qs = qs.filter(
                    delivery_date__gte=first_date,
                    delivery_date__lte=last_date
                )                    
            elif delivery == 's':
                # semestrale: 1-2-3, 4-5-6, 7-8-9, 10-11-12
                today = datetime.date.today()
                first_month = (today.month - 1) / 6 * 6 + 1
                last_month = first_month + 5
                last_day = calendar.monthrange(today.year, last_month)[1]
                first_date = datetime.date(today.year, first_month, 1)
                last_date = datetime.date(today.year, last_month, last_day)
                qs = qs.filter(
                    delivery_date__gte=first_date,
                    delivery_date__lte=last_date
                )                    
            elif delivery == 'a':
                qs = qs.filter(
                    delivery_date__year=datetime.date.today().year
                )
            del lookup_params['delivery']


        return lookup_params, qs

class PressSearchFormConsumer(PressSearchForm):
    class Meta:
        model = Press
        display_fields = (
            ('date_p__gte', 'date_p__lte'),
            'delivery',
            'source__name__icontains',
            ('press_reviews__name__icontains', 'themes__name__icontains'),
            'press_reviews__slug',
            ('only_title', 'title_abstract')
        )
       
class PressAdmin(ExtendibleModelAdmin):
    save_on_top = True
    list_display_super = (
        'action_checkbox', 'get_status_icon', 'date_p', 'title', 'source', 'get_list_themes',
        'abstract', 'note', 'get_list_pressreviews',
        'get_link_file',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    list_display_guest = (
        'date_p', 'title', 'source', 'get_list_themes',
        'abstract', 'note', 'get_list_pressreviews',
        'get_link_file',
    )
    list_display_csv = (
        'date_p', 'title', 'source', 'get_list_themes',
        'get_abstract_csv', 'get_note_csv', 'get_list_pressreviews',
        'get_link_file_csv',
    )
    list_display = list_display_super
    ordering_guest = ('-date_p',)

    search_fields = ('title',)
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create','last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'date_p', 'delivery_date', 'title', 'slug', 'source',
            'themes', 'press_reviews',
            'abstract', 'keywords', 'file', 'note'
        ),}),
    )
    prepopulated_fields = {"slug": ("title",)}

    def get_search_form(self):
        user = get_current_user()
        if user.is_superuser:
            return PressSearchForm
        else:
            return PressSearchFormConsumer

    def get_abstract_csv(self, obj):
        abstract = obj.abstract.replace(';', ' - ')
        return abstract

    def get_note_csv(self, obj):
        note = obj.note.replace(';', ' - ')
        return note

    def get_link_file_csv(self, obj):
        if obj.can_be_downloaded():
            return "office.knba.it%s" % obj.file.url
        return ""
                                                                
    def get_title(self, obj):
        return "<b>%s</b>" % obj.title
    get_title.short_description = _("Title")
    get_title.admin_order_field = 'title'
    get_title.allow_tags = True

    def get_actions(self, request):
        if not request.user.is_superuser:
            return []
        return super(PressAdmin, self).get_actions(request)

    def changelist_view(self, request, extra_context=None):
        user = request.user
        if not user.is_superuser:
            press_review = None
            pressreview_slug = request.GET.get('press_reviews__slug' or None)
            reviews = PressReview.objects.all()
            if pressreview_slug:
                reviews = reviews.filter(slug=pressreview_slug)

            if reviews:
                press_review = reviews[0]

            if press_review:
                list_fields = press_review.list_fields
                ordering_list = press_review.ordering_list
                if list_fields:
                    self.list_display = list_fields.split(',')
                else:
                    self.list_display = self.list_display_guest
                if ordering_list:
                    self.ordering = ordering_list.split(',')
                else:
                    self.ordering = self.ordering_guest
        else:
            self.list_display = self.list_display_super

        response = super(PressAdmin, self).changelist_view(request, extra_context)
        return response

    def queryset(self, request):
        qs = super(PressAdmin, self).queryset(request)
        user = request.user
        if user.is_superuser == False:
            press_review = None
            pressreview_slug = request.GET.get('press_reviews__slug' or None)
            reviews = PressReview.objects.all()
            if pressreview_slug:
                reviews = reviews.filter(slug=pressreview_slug)
            if reviews:
                press_review = reviews[0]

            if press_review:
                qs = qs.filter(press_reviews=press_review)
        return qs

    def get_link_file(self, obj):
        if obj.can_be_downloaded():
            return """
                <a target="_blank" href="%(url)s" title="%(title)s">
                    <img src="%(media_url)sadmin/images/press.png" alt="%(description)s" title="%(description)s"/>
                </a>
            """ % {
                'url'         : obj.file.url,
                'title'       : obj.file.name,
                'media_url'   : settings.MEDIA_URL,
                'description' : _("Download Press")
            }
        return ""
    get_link_file.short_description = _("P")
    get_link_file.allow_tags = True

    #ordering = ('title',)
    def my_view(self, request):
        # custom view which should return an HttpResponse
        pass

class SourceAdmin(ExtendibleModelAdmin):
    model = Source
    save_on_top = True
    list_display = (
        'get_status_icon', 'get_periodicity_display', 'sector', 'list_formats', 'name',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    search_fields = ('name',)
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create','last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', ('periodicity', 'sector',), 'formats', ('name', 'slug',), 'description',
        ),}),
    )
    prepopulated_fields = {"slug": ("name",)}

class SectorAdmin(ExtendibleModelAdmin):
    model = Sector
    save_on_top = True
    list_display = (
        'get_status_icon', 'name',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    search_fields = ('name',)
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create','last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'name',
        ),}),
    )

class FormatAdmin(ExtendibleModelAdmin):
    model = Format
    save_on_top = True
    list_display = (
        'get_status_icon', 'name',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    search_fields = ('name',)
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create','last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'name',
        ),}),
    )

class ThemeAdmin(ExtendibleModelAdmin):
    model = Theme
    save_on_top = True
    list_display = (
        'get_status_icon', 'name',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    search_fields = ('name',)
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create','last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'name', 'slug', 'description',
        ),}),
    )
    prepopulated_fields = {"slug": ("name",)}

class PressReviewAdmin(ExtendibleModelAdmin):
    model = PressReview
    save_on_top = True
    list_display = (
        'get_status_icon', 'name', 'get_list_organizations',
        'delivery', 'ordering_list', 'list_fields',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    search_fields = ('name',)
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create','last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'name', 'slug', 'organizations', 'ordering_list',
            'list_fields', 'delivery', 'description',
        ),}),
    )
    filter_horizontal = ('organizations',)
    prepopulated_fields = {"slug": ("name",)}

admin.site.register(Press, PressAdmin)
admin.site.register(Source, SourceAdmin)
admin.site.register(Format, FormatAdmin)
admin.site.register(Sector, SectorAdmin)
admin.site.register(Theme, ThemeAdmin)
admin.site.register(PressReview, PressReviewAdmin)
